@@ -1,5 +1,5 @@
 /*
-  Copyright 2012-2018 Jyri J. Virkki <jyri@virkki.com>
+  Copyright 2012-2023 Jyri J. Virkki <jyri@virkki.com>
 
   This file is part of dupd.
 
@@ -383,7 +383,7 @@ void operation_report()
  *
  * Parameters:
  *    dbh  - sqlite3 database handle.
- *    size - IGNORED
+ *    size - Size of the file, but note it may be SCAN_SIZE_UNKNOWN
  *    path - Process this file.
  *
  * Return:
@@ -403,6 +403,26 @@ static int file_callback(sqlite3 * dbh,
   char * unique_pfx = "";
   char * dup_pfx = "";
 
+  // If the file is smaller than what the scan cared about, ignore it.
+  // Otherwise it leads to inconsistent behavior where a file is not
+  // processed during the scan (and thus not reported by report) but
+  // is reported by the interactive functions. This has led to confusion,
+  // so it is best to be consistent in output.
+
+  if (size == SCAN_SIZE_UNKNOWN) {
+    STRUCT_STAT stat_info;
+    int rv = get_file_info(path, &stat_info);
+    if (rv != 0) {
+      printf("error: unable to stat the size of %s\n", path);
+      exit(1);
+    }
+    size = (uint64_t)stat_info.st_size;
+  }
+
+  if (size < minimum_file_size) {
+    return 0;
+  }
+
   // For uniques and dups, only print the list of filenames.
   // For ls which prints both, include prefixes to identify the type.
   if (print_uniques && print_duplicates) {
@@ -415,7 +435,12 @@ static int file_callback(sqlite3 * dbh,
     if (is_known_unique(dbh, path)) {
       if (print_uniques) {
         print_path(unique_pfx, path);
+        return(0);
       }
+    }
+    // If we're only printing uniques and we have_uniques above, nothing
+    // more left to do for this file.
+    if (!print_duplicates) {
       return(0);
     }
   }
@@ -508,18 +533,34 @@ void operation_ls()
  */
 void operation_uniques()
 {
-  print_uniques = 1;
-  LOG_MORE {
-    list_all_duplicates = 1;
-  }
+  if (have_uniques) {
 
-  sqlite3 * dbh = open_database(db_path, 0);
-  init_get_known_duplicates();
-  init_scanlist();
-  init_dirtree();
-  walk_dir(dbh, start_path[0], NULL, 0, file_callback);
-  close_database(dbh);
-  free_get_known_duplicates();
+    // If we have_uniques (the files table) in the db, simply print out
+    // all the unique files (at time of scan) which fall under start_path[0]
+
+    sqlite3 * dbh = open_database(db_path, 0);
+    print_all_uniques(dbh, start_path[0]);
+    close_database(dbh);
+    return;
+
+  } else {
+
+    // Otherise, walk the dir tree and identify uniques by their absensce in
+    // duplicates table, which is (much) slower.
+
+    print_uniques = 1;
+    LOG_MORE {
+      list_all_duplicates = 1;
+    }
+
+    sqlite3 * dbh = open_database(db_path, 0);
+    init_get_known_duplicates();
+    init_scanlist();
+    init_dirtree();
+    walk_dir(dbh, start_path[0], NULL, 0, file_callback);
+    close_database(dbh);
+    free_get_known_duplicates();
+  }
 }
 
 
